--[[
Craftaholic 
Version: Beta 1.02
Game Version: 3.2.0a
Copyright 2009 Alex DiMarzo
Thegreatiam, Hyjal

This file is part of Craftoholic.

Craftoholic is free software: you can redistribute it and/or modify
it under the terms of version 3 of the GNU General Public License 
as published by the Free Software Foundation.

Craftoholic is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Craftoholic.  If not, see <http://www.gnu.org/licenses/>.
]]--

-- Set varibles for the script; currently global until GUI is added, then they can be made local.
CH_MaxResponses = 35
CH_REALMNAME = GetRealmName()
CH_PLAYERNAME = UnitName("player")

-- Localization Variables :: Should move to localization file eventually.
local CHL_HelpResponseA = 'CH: For a full profession link, whisper with !full followed by all or part of a profession name '..
				'(EG: !full jewelcrafting, !full eng) or for all available profession links use !all or !full all.'

local CHL_HelpResponseB = 'CH: For an individual craft link, whisper with !profession <stats, item name, etc. (EG: !enchant shield defense, !jc stam str).'

local CHL_Returned = "Returned"
local CHL_ForChar = "link(s) for the character"

-- Create tables for later use; could use "if not..." instead?
if CH_ProfessionTable == nil then CH_ProfessionTable = {} end
if CH_ProfessionTable[CH_REALMNAME] == nil then CH_ProfessionTable[CH_REALMNAME] = {} end
if CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME] == nil then CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME] = {} end
local CH_Professions = {}

local function CH_Respond(MSG,TAR) -- Simplifies the process of sending responses
	SendChatMessage(MSG,"WHISPER",nil,TAR)
end

local function CH_OMGHide(self,event,xmsg,TAR) -- Filters out Craftoholic-sent messages and prevents double-printing when whispering yourself.
	msg = strlower(xmsg)
		if (select(2,strfind(msg,"ch: ")) == 4) then
			return true;
		elseif TAR == CH_PLAYERNAME then
			return true;
		else
			return false;
		end -- ==> if (select(...
end -- ==> function CH_OMGHide

local function CH_GetCharProfs() -- Puts the current character's professions into the table CH_Professions
	local NumSkillLines = GetNumSkillLines()
	for SkillIndex = NumSkillLines,1,-1 do
		CollapseSkillHeader(SkillIndex)
	end
	ExpandSkillHeader(2)
	CH_Professions = {GetSkillLineInfo(3),GetSkillLineInfo(4)}
	for SkillIndex = NumSkillLines,1,-1 do
		ExpandSkillHeader(SkillIndex)
	end
end -- ==> function CH_GetCharProfs

function CH_ProfScan()
	local Profession,Skill,Max = GetTradeSkillLine()
	
	-- Removes saved info from deleted professions by waiting until a link is viewed; have not set up an OnEvent for it yet.
	if GetSpellLink(Profession) == nil then
		if CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession] then
			CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession] = nil
		end
		return 1
	end
	
	local ListLink = GetTradeSkillListLink()
	if IsTradeSkillLinked() == 1 then return 1 end
	if Profession ~= "Mining" then if ListLink == nil then return 1 end end
	
	-- Create tables if needed
	if CH_ProfessionTable == nil then CH_ProfessionTable = {} end
	if CH_ProfessionTable[CH_REALMNAME] == nil then CH_ProfessionTable[CH_REALMNAME] = {} end
	if CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME] == nil then CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME] = {} end
	if CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession] == nil then CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession] = {} end
		
	-- Enter values into appropriate table
	local CH_VarTable = CH_ProfessionTable[CH_REALMNAME][CH_PLAYERNAME][Profession]
	CH_VarTable[1]=Profession
	CH_VarTable[2]=Skill
	CH_VarTable[3]=Max
	
		-- Determine if character is specialized in their profession and use appropriate link.

		local CH_Elixir = select(2,GetSpellLink("Elixir Master"))
		local CH_Potion = select(2,GetSpellLink("Potion Master"))
		local CH_Xmute = select(2,GetSpellLink("Transmutation Master"))
		
		local CH_Goblin = select(2,GetSpellLink("Goblin Engineer"))
		local CH_Gnomish = select(2,GetSpellLink("Gnomish Engineer"))
		
		local CH_Armor = select(2,GetSpellLink("Armorsmith"))
		local CH_Weapon = select(2,GetSpellLink("Weaponsmith"))
		local CH_Axes = select(2,GetSpellLink("Master Axesmith"))
		local CH_Hammers = select(2,GetSpellLink("Master Hammersmith"))
		local CH_Swords = select(2,GetSpellLink("Master Swordsmith"))
		
		local CH_Elemental = select(2,GetSpellLink("Elemental Leatherworking"))
		local CH_Dragonscale = select(2,GetSpellLink("Dragonscale Leatherworking"))
		local CH_Tribal = select(2,GetSpellLink("Tribal Leatherworking"))
		
		local CH_Spell = select(2,GetSpellLink("Spellfire Tailoring"))
		local CH_Shadow = select(2,GetSpellLink("Shadoweave Tailoring"))
		local CH_Moon = select(2,GetSpellLink("Mooncloth Tailoring"))
		
		if Profession == "Alchemy" then
			if		CH_Elixir ~= nil then CH_VarTable[4] = CH_Elixir
			elseif	CH_Potion ~= nil then CH_VarTable[4] = CH_Potion
			elseif	CH_Xmute ~= nil then CH_VarTable[4] = CH_Xmute
			else	CH_VarTable[4] = select(2,GetSpellLink(Profession))
			end		
		elseif Profession == "Tailoring" then
			if		CH_Spell ~= nil then CH_VarTable[4] = CH_Spell
			elseif	CH_Shadow ~= nil then CH_VarTable[4] = CH_Shadow
			elseif	CH_Moon ~= nil then CH_VarTable[4] = CH_Moon
			else	CH_VarTable[4] = select(2,GetSpellLink(Profession))
			end
		elseif Profession == "Leatherworking" then
			if		CH_Elemental ~= nil then CH_VarTable[4] = CH_Elemental
			elseif	CH_Dragonscale ~= nil then CH_VarTable[4] = CH_Dragonscale
			elseif	CH_Tribal ~= nil then CH_VarTable[4] = CH_Tribal
			else	CH_VarTable[4] = select(2,GetSpellLink(Profession))
			end
		elseif Profession == "Blacksmithing" then
			if		CH_Armor ~= nil then CH_VarTable[4] = CH_Armor
			elseif	CH_Axes ~= nil then CH_VarTable[4] = CH_Axes
			elseif	CH_Hammers ~= nil then CH_VarTable[4] = CH_Hammers
			elseif	CH_Swords ~= nil then CH_VarTable[4] = CH_Swords
			elseif	CH_Weapon ~= nil then CH_VarTable[4] = CH_Weapon
			else	CH_VarTable[4] = select(2,GetSpellLink(Profession))
			end		
		elseif Profession == "Engineering" then
			if		CH_Goblin ~= nil then CH_VarTable[4] = CH_Goblin
			elseif	CH_Gnomish ~= nil then CH_VarTable[4] = CH_Gnomish
			else	CH_VarTable[4] = select(2,GetSpellLink(Profession))
			end		
		elseif Profession ~= "Mining" then
			CH_VarTable[4]=select(2,GetSpellLink(Profession))
		end

	-- Clear current Recipe DB
	CH_VarTable["Recipes"] = {} 
	
	-- Expand all headers so that nothing gets skipped; should already be done per WoW behavior but you never know.
	for i=GetNumTradeSkills(),1,-1 do
		if GetTradeSkillItemLink(i) == nil then
			ExpandTradeSkillSubClass(i)
		end
	end
	
	-- :: Scan and record tooltip ::
	for i=1,GetNumTradeSkills() do
		if GetTradeSkillItemLink(i) ~= nil then
			local WinString = GetTradeSkillInfo(i)
			CH_TooltipScan:ClearLines()
			CH_TooltipScan:SetHyperlink(GetTradeSkillItemLink(i))
			for i=1,CH_TooltipScan:NumLines() do
			   local scantext = getglobal("CH_TooltipScanTextLeft" .. i)
			   local utext = scantext:GetText()
			   local atext = gsub(utext,"|cffffffffrequires","requires")
			   local btext = gsub(atext,"|cff808080requires","requires")
			   local ctext = gsub(btext,"|r\n","")
			   local text = gsub(ctext,"|r","")
				WinString = WinString.." "..strlower(text)
			end -- ==> for i-1,CH_
			CH_VarTable["Recipes"][WinString] = GetTradeSkillRecipeLink(i)
		end -- ==> if GetTradeSkillItemLink
	end -- == for i=1,GetNumTradeSkills()
end -- ==> function CH_ProfScan()

local function CH_FormatSearch(xstring) -- Used by CraftyResponse to match a user's query to a profession.
	local string = strlower(xstring)
	
	if	   string == nil	then return "help"
	elseif string == ""		then return "help"
	elseif string == "help"	then return "help"
	elseif string == "halp" then return "help"
	elseif string == "all"	then return "all"
	elseif string == "full"	then return "full"
	elseif string == "ing"	then return "specific"
	
	elseif strmatch("alchemy alchemist",string)					then return "Alchemy"
	elseif strmatch("bs blacksmithing",string)					then return "Blacksmithing"
	elseif strmatch("ck cooking chef nomnomnomnom",string)		then return "Cooking"
	elseif strmatch("enchanting enchanter",string)				then return "Enchanting"
	elseif strmatch("eg engineering",string)					then return "Engineering"
	elseif strmatch("scribe inscription glyph",string)			then return "Inscription"
	elseif strmatch("jc gem jewelcrafting jewelcrafter",string)	then return "Jewelcrafting"
	elseif strmatch("lw leatherworking leatherworker",string)	then return "Leatherworking"
	elseif strmatch("smelting mining miner ",string)			then return "Mining"
	elseif strmatch("tailoring",string)							then return "Tailoring"
	------------------------------------
	----- *ADD* (maybe) First Aid ------
	------------------------------------

	else return nil
	end -- ==> if string == nil
end -- ==> function CH_FormatSearch

local function CH_FullQuery(profession)  -- Matches query to list of professions for each character on the realm.
local CharString = "" local NumResponses = 0 local ResponseTable = {} local RealmTable = CH_ProfessionTable[CH_REALMNAME]
	for Character,_ in pairs(RealmTable) do
		local CharacterTable = RealmTable[Character]
		for ProfName,_ in pairs(CharacterTable) do
			local ProfTable = CharacterTable[ProfName]
			if profession == "all" then
				if ProfName ~= "First Aid" then
					if ProfName ~= "Cooking" then
						if ProfTable[4] ~= nil then
						NumResponses = NumResponses + 1
						local LINK = ProfTable[4]
						local CurrentSkill = ProfTable[2]
						local MaxSkill = ProfTable[3]
						local ResponseLine = "CH: " .. Character .. " - " .. LINK .. " " .. CurrentSkill .. "/" .. MaxSkill
						tinsert(ResponseTable,ResponseLine)
						if CharString == "" then CharString = Character else if not strmatch(CharString,Character) then CharString = CharString..", "..Character end end
						end -- ==> if ProfTable[4] ~= nil
					end -- ==> if ProfName ~= "Cooking"
				end -- ==> if ProfName ~= "First Aid"
			elseif ProfName == profession then
					NumResponses = NumResponses + 1
					local LINK = ProfTable[4]
					local CurrentSkill = ProfTable[2]
					local MaxSkill = ProfTable[3]
					local ResponseLine = "CH: " .. Character .. " - " .. LINK .. " " .. CurrentSkill .. "/" .. MaxSkill
					tinsert(ResponseTable,ResponseLine)
					if CharString == "" then CharString = Character else if not strmatch(CharString,Character) then CharString = CharString..", "..Character end end
			end -- ==> if ProfName == profession
		end -- ==> for ProfName,_
	end -- ==> for Character,_
return ResponseTable, NumResponses, CharString
end -- ==> function CH_FullQuery

local function CH_GetMatches(profession,query)  -- Matches query to tooltip text for a given profession.
local CharString = ""
local STRINGTABLE = {} local RealmTable = CH_ProfessionTable[CH_REALMNAME]
local CH_TotalResponses = 0
local CH_QueryResponseTable = {}
local CH_ReturnedLinkTable = {}

string.gsub(query, "([%w%p]+)" , function(x) table.insert(STRINGTABLE,x) return x end)
	
	for Character,_ in pairs(RealmTable) do
		local CharacterTable = RealmTable[Character]
		
		for ProfName,_ in pairs(CharacterTable) do
			local ProfTable = CharacterTable[ProfName]
			
			if ProfName == profession then

				RecipeTable = ProfTable["Recipes"]
				if RecipeTable then
				
					for CachedText,RecipeLink in pairs(RecipeTable) do 
						SearchText = strlower(CachedText)
						local NumMatches = 0
						
						for _,QueryText in pairs(STRINGTABLE) do 
							if strmatch(SearchText, QueryText) == QueryText then
								NumMatches = NumMatches + 1
								
							end
						end -- ==> for _,QueryText
						
						if NumMatches == #(STRINGTABLE) then
							if not CH_ReturnedLinkTable[Character] then
								CH_ReturnedLinkTable[Character] = {}
							end
							
							CH_ReturnedLinkTable[Character][RecipeLink] = ""
							
						end -- ==> if NumMatches...
						
					end -- ==> for CachedText...
				end -- ==> if RecipeTable then
			end -- ==> if ProfName == profession
		end -- ==> for Profname,_ in pairs
	end -- ==> for Character,_ in pairs
	
	for CharacterName,Table in pairs(CH_ReturnedLinkTable) do
		if CharString == "" then CharString = CharacterName else CharString = CharString..", "..CharacterName end
		local n=0 for k,v in pairs(Table) do n=n+1 end CH_CharResponses=n
		CH_TotalResponses = CH_TotalResponses + CH_CharResponses
		tinsert(CH_QueryResponseTable,CHL_Returned.." "..CH_CharResponses.." "..CHL_ForChar.." "..CharacterName..":")
		for Link,Comment in pairs(Table) do
			if Comment ~= "" then
				tinsert(CH_QueryResponseTable,Link.." - "..Comment)
			else
				tinsert(CH_QueryResponseTable,Link)
			end
		end
	end
	
	return CH_QueryResponseTable, CH_TotalResponses, CharString
end -- ==> function CH_GetMatches

local function CH_GetResponse(msg,customer)
	local _,_,flag,rtype,qtext = strfind(msg, "(%p)(%w+)[%s%p]*(.*)")

	if flag == "!" then
		if rtype then
			if CH_FormatSearch(rtype) == nil then
				CH_Respond(CHL_HelpResponseA,customer)
				CH_Respond(CHL_HelpResponseB,customer)
			elseif CH_FormatSearch(rtype) == "full" then
				local query = CH_FormatSearch(qtext)
				local ResponseTable,NumResponses,CharString = CH_FullQuery(query)
				if NumResponses == 0 then 
				print(customer.." received no responses for "..xmsg) else print("Returned "..NumResponses.." links for the characters: "..CharString)
				end
				for _,value in pairs(ResponseTable) do
					CH_Respond(value,customer)
				end
			elseif CH_FormatSearch(rtype) == "all" then
				local ResponseTable,NumResponses,CharString = CH_FullQuery(rtype)
				if NumResponses == 0 then 
				print(customer.." received no responses for "..xmsg) else print("Returned "..NumResponses.." links for the characters: "..CharString)
				end
				for _,value in pairs(ResponseTable) do
					CH_Respond(value,customer)
				end
			elseif CH_FormatSearch(rtype) == "halp" then
				CH_Respond('CH: For a full profession link, whisper with !full followed by all or part of a profession name (EG: !full jewelcrafting, !full eng) or for all available profession links use !all or !full all.',customer)
				CH_Respond('CH: For an individual craft link, whisper with !profession <stats, item name, etc> (EG: !enchant shield defense, !jc stam str).',customer)
			else
				local profession = CH_FormatSearch(rtype)
				local ResponseTable,NumResponses,CharString = CH_GetMatches(profession,qtext)
				if NumResponses == 0 then 
				print(customer.." received no responses for "..xmsg) else print("Returned "..NumResponses.." links for the characters: "..CharString)
				end
				for _,value in pairs(ResponseTable) do
					CH_Respond(value,customer)
				end
			
			end -- ==> if CH_FormatSearch(rtype) == nil
		end -- ==> if rtype then
	end -- ==> if flag == "!"
end -- ==> function CH_GetResponses


local function CH_CraftyResponse(self,event,...)  -- Event handler
	if event == "CHAT_MSG_WHISPER" then
		xmsg,customer = ...
		msg = strlower(xmsg)
			local start,_,flag,rtype,qtext = strfind(msg, "(%p)([%w%p]+)[%s%p]*(.*)")  -- rtype, qtext, customer
		if start == 1 then if flag == "!" then if rtype then
			local CH_SearchBy = CH_FormatSearch(rtype)
			if qtext then local CH_SearchFor = CH_FormatSearch(qtext) end
			
			if CH_SearchBy == "help" then
				CH_Respond(CHL_HelpResponseA,customer)
				CH_Respond(CHL_HelpResponseB,customer)
			
			elseif CH_SearchBy == "all" or CH_SearchFor == "all" then
				local ResponseTable,NumResponses,CharString = CH_FullQuery(rtype)
				if NumResponses == 0 then 
					print(customer.." received no responses for "..xmsg)
				elseif NumResponses < CH_MaxResponses then
					print("Returned "..NumResponses.." links for the characters: "..CharString)
					for _,value in pairs(ResponseTable) do
						CH_Respond(value,customer)
					end
				else
					CH_Respond("CH: Your search returned too many results.",customer)
				end
				
			elseif CH_SearchBy == "full" then -- == *** TO DO *** find a way to combine this condition with the next one
				
				
				local query = CH_FormatSearch(qtext)
				local ResponseTable,NumResponses,CharString = CH_FullQuery(query)
				if NumResponses == 0 then 
					print(customer.." received no responses for "..xmsg)
				elseif NumResponses < CH_MaxResponses then
					print("Returned "..NumResponses.." link(s) for the characters: "..CharString)
					for _,value in pairs(ResponseTable) do
						CH_Respond(value,customer)
					end
				else
					CH_Respond("CH: Your search returned too many results.",customer)
				end
			

			
			else
				local profession = CH_SearchBy
				local ResponseTable,NumResponses,CharString = CH_GetMatches(profession,qtext)
				if NumResponses == 0 then 
					print(customer.." received no responses for "..xmsg)
				elseif NumResponses < CH_MaxResponses then
					print("Returned "..NumResponses.." link(s) for the characters: "..CharString)
					for _,value in pairs(ResponseTable) do
						CH_Respond(value,customer)
					end
				else
					CH_Respond("CH: Your search returned too many results.",customer)
				end
				
			end -- ==> if CH_FormatSearch(rtype) == nil
		end -- ==> if rtype then
		end -- ==> if flag == "!"
		end -- ==> if start == 1

	elseif event == "TRADE_SKILL_SHOW" then
		CH_GetCharProfs()
		CH_ProfScan()
		
		
	end -- ==> if event == "CHAT_MSG_WHISPER"
 end -- ==> function CraftyResponse
 

CH_EventFrame = CreateFrame("FRAME","CH_Event_Frame")
CH_EventFrame:RegisterEvent("CHAT_MSG_WHISPER")
CH_EventFrame:RegisterEvent("TRADE_SKILL_SHOW")
CH_EventFrame:SetScript("OnEvent",CH_CraftyResponse)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM",CH_OMGHide)